@@ -1,14 +1,16 @@
 package Gearman::Job;
-use version;
-$Gearman::Job::VERSION = qv("1.130.004");
+use version ();
+$Gearman::Job::VERSION = version->declare("2.002.003");
 
 use strict;
 use warnings;
 
-#TODO: retries?
-#
-use Gearman::Util;
+use Gearman::Util ();
 use Carp ();
+use Ref::Util qw/
+    is_plain_scalarref
+    is_ref
+    /;
 
 =head1 NAME
 
@@ -31,9 +33,10 @@ use fields (
 );
 
 sub new {
-    my ($class, $func, $argref, $handle, $jss) = @_;
-    my $self = $class;
-    $self = fields::new($class) unless ref $self;
+    my ($self, $func, $argref, $handle, $jss) = @_;
+    unless (is_ref($self)) {
+        $self = fields::new($self);
+    }
 
     $self->{func}   = $func;
     $self->{handle} = $handle;
@@ -51,7 +54,7 @@ represent the percentage completion of the job.
 =cut
 
 sub set_status {
-    my Gearman::Job $self = shift;
+    my $self = shift;
     my ($nu, $de) = @_;
 
     my $req = Gearman::Util::pack_req_command("work_status",
@@ -68,7 +71,7 @@ sub set_status {
 =cut
 
 sub argref {
-    my Gearman::Job $self = shift;
+    my $self = shift;
     return $self->{argref};
 }
 
@@ -79,7 +82,7 @@ B<return> the scalar argument that the client sent to the job server.
 =cut
 
 sub arg {
-    my Gearman::Job $self = shift;
+    my $self = shift;
     return ${ $self->{argref} };
 }
 
@@ -90,7 +93,7 @@ B<return> handle
 =cut
 
 sub handle {
-    my Gearman::Job $self = shift;
+    my $self = shift;
     return $self->{handle};
 }
 
